*-------------------------------------------------------------------
*
* Name.........: HHMAIN.PRG
* Purpose......: Entry point to Home Hunters' Information Manager 
* Function.....: Calls HHMAIN.MPR and issues foundation read
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
IF .f.
  ? "(C) 1995 by Ned Epstein, All Rights Reserved."
ENDIF
deactivate windows all && development 
clear all && development
close databases
DO HHWELCOM.SPR 
ON KEY LABEL CTRL+F2 DO lpDebug 
ON KEY LABEL ESC DO lpTerm 
DO lpSetEnv
DO lpDefine
DO lpSetUp
DO HHMENU.MPR
READ VALID TaskLauncher() && Foundation Read
WAIT WINDOW [End of Processing] NOWAIT
SET SYSMENU TO DEFAULT
DO lpCleanUp
RETURN

*-------------------------------------------------------------------
*
* Name.........: lpCleanUp
* Purpose......: Release all memory variable and close open tables
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE lpCleanUp
  RELEASE a_HHSite, a_HHAgen 
  RELEASE gcTables, llQuitRead
  RELEASE glSiteOpt, glAgenOpt 
  RELEASE lcLaunchIt, llTerminate
  RELEASE gaSysTask 
  RELEASE a_Tmpf1, a_Tmpf2, a_Tmpf3, a_Tmpf4, a_Tmpf5, a_Tmpf6
  RELEASE a_Tcur1, a_Tcur2, a_Tcur3, a_Tcur4, a_Tcur5, a_Tcur6
  RELEASE ALL
  CLOSE DATABASES
RETURN

*-------------------------------------------------------------------
*
* Name.........: lpDefine
* Purpose......: Define global constants and arrays 
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE lpDefine

  PUBLIC a_HHSite, a_HHAgen 
  PUBLIC gcTables, llQuitRead
  PUBLIC glSiteOpt, glAgenOpt 
  PUBLIC lcLaunchIt, llTerminate

  *-- Task array
  #DEFINE dcMaxTasks 2 && Maximum number of tasks currently supported
  #DEFINE dcMaxVars  7 && Variables to define task state
  PUBLIC ARRAY gaSysTask(dcMaxTasks,dcMaxVars)
  *-- define task state constants 
  #DEFINE dcTaskName 1 && Task name is stored in column 1
  #DEFINE dcWName    2 && Window name stored in column 2
  #DEFINE dcCurRec   3 && Current record of suspended task in column 3
  #DEFINE dcSuspend  4 && Window release switch in column 4 
  #DEFINE dcSEdit    5 && Edit mode switch in column 5 
  #DEFINE dcSAdd     6 && Add mode switch in column 6 
  #DEFINE dcCurObj   7 && Current object
  *-- Initialize task array
  *-- By default all elements are initialized to .f. so skip 
  *-- dcsadd, dcsedit, dcsuspend.
  STORE [HHSITE]  TO gaSysTask(1,dcTaskName) 
  STORE [HHAGEN]  TO gaSysTask(2,dcTaskName) 
  STORE [WHHSITE] TO gaSysTask(1,dcWName) 
  STORE [WHHAGEN] TO gaSysTask(2,dcWName) 
  STORE 0         TO gaSysTask(1,dcCurRec) && default to top of file
  STORE 0         TO gaSysTask(2,dcCurRec) && default to top of file
  *-- Current object 
  STORE 0        TO gaSysTask(1,dcCurObj) 
  STORE 0        TO gaSysTask(2,dcCurObj) 
  
  *-- Temporary table
  PUBLIC a_Tmpf1, a_Tmpf2, a_Tmpf3, a_Tmpf4, a_Tmpf5, a_Tmpf6
  *-- Temporary cursor
  PUBLIC a_Tcur1, a_Tcur2, a_Tcur3, a_Tcur4, a_Tcur5, a_Tcur6

  *-- Enable Menu Options on Other popup
  STORE .f. TO glSiteOpt, glAgenOpt 
  STORE []  TO lcLaunchIt 
  STORE .f. TO llTerminate

  STORE [HHsite] TO a_HHSite
  STORE [HHagen] TO a_HHAgen

  STORE [00001111] TO a_Tmpf1
  STORE [00002222] TO a_Tmpf2
  STORE [00003333] TO a_Tmpf3
  STORE [00004444] TO a_Tmpf4
  STORE [00005555] TO a_Tmpf5
  STORE [00006666] TO a_Tmpf6

  STORE [00007777] TO a_Tcur1
  STORE [00008888] TO a_Tcur2
  STORE [00009999] TO a_Tcur3
  STORE [00001010] TO a_Tcur4
  STORE [00001212] TO a_Tcur5
  STORE [00001313] TO a_Tcur6

RETURN

*-------------------------------------------------------------------
*
* Name.........: lpSetEnv
* Purpose......: Set environment 
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE lpSetEnv
  SET CURSOR ON
  SET STATUS BAR ON
RETURN

*-------------------------------------------------------------------
*
* Name.........: lpSetUp
* Purpose......: Open tables and define aliases
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE lpSetUp
*-- Open Broker table
SELECT 0
USE (a_HHSite) ALIAS Site 
SET ORDER TO TAG SCode
*-- Open Commodity table
SELECT 0
USE (a_HHAgen) ALIAS Agen 
SET ORDER TO TAG ACode

RETURN

*-------------------------------------------------------------------
*
* Name.........: lpTerm
* Purpose......: Terminate processing
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE lpTerm 
* The order of these two statements is important since
* issuing a CLEAR READ ALL when a READ is issued with
* no GETS causes the READ's VALID clause to be executed 
  STORE .t. TO llTerminate
  RELEASE WINDOW wRestart
  CLEAR READ ALL
RETURN

*-------------------------------------------------------------------
*
* Name.........: lpDebug
* Purpose......: Set up environment for debugging
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE lpDebug
  ACTIVATE WINDOW COMMAND
  ACTIVATE WINDOW DEBUG
  ACTIVATE WINDOW TRACE
  SET STEP ON
RETURN

*-------------------------------------------------------------------
*
* Name.........: TaskLauncher
* Purpose......: Foundation read. If CLEAR READ is ussued this 
*                logic will be executed. If lcLaunchIt was set 
*                then the program is run. Otherwise the array 
*                SysTask is checked to see if any of the windows 
*                defined for this application are open. If so
*                the task associated with the top-most window 
*                is dispatched.
*                The Foundation READ terminates when this routine 
*                returns .T.  As long as it returns .F., execution 
*                of the Foundation READ will continue. 
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
FUNCTION TaskLauncher
PRIVATE lcTask

IF llTerminate
   RETURN .T. && When menu 'Exit' is selected
ENDIF

* See comment below for definition of wRestart
IF WEXIST("wRestart") 
  HIDE WINDOW wRestart
ENDIF

* check if new task launched from from menu
IF LEN(lcLaunchIt) > 0
   lcTask = lcLaunchIt	&& Start it ON TOP of current screens
   lcLaunchIt = []
   DO (LcTask)
   RETURN .F.
ENDIF

* What if an application was exited or windows switched?
* Search for next task and launch it
lcLaunchIt=[]
PRIVATE lnCounter
FOR lnTask=1 TO dcMaxTasks && find topmost screen of dispatched tasks
  IF WONTOP(ALLTRIM(gaSysTask(lnTask,dcWName))) 
    lcLaunchIt=ALLTRIM(gaSysTask(lnTask,dcTaskName))
    EXIT 
  ENDIF
ENDFOR
IF LEN(lcLaunchIt) > 0   
   lcTask = lcLaunchIt	&& Start it ON TOP of current screens
   lcLaunchIt = []
   DO (LcTask)
   RETURN .F.
ENDIF
*
* No tasks to launch 
*
* This window is for waking up the foundation read when all tasks are 
* closed.  It re-enables the QUIT option on the main menu. 
IF !WEXIST("wRestart")
  DEFINE WINDOW wRestart AT -10,-10 SIZE 1,1 NONE
ENDIF
ACTIVATE WINDOW "wRestart"
RETURN .F.

*-------------------------------------------------------------------
*
* Name.........: MenuTask
* Purpose......: To set the variable lcLanchIt with the program name 
*                selected from the menu. It also runs the DEACTIVATE
*                routine of any active task and issues a CLEAR READ 
*                for the task.
* Called by....: HHMAIN.MPR
* Author.......: Ned Epstein
* Date.........: 05/30/95
*
*-------------------------------------------------------------------
PROCEDURE MenuTask
PARAMETER lcTask
  lcLaunchIt = lcTask && store new task name to pass to launch routine
  IF RDLEVEL() > 1
    * Save the current tasks environment, 
    * execute its DEACTIVATE routine 
    =EVALUATE("lpRD()")
  ENDIF
  CLEAR READ && CLEAR READ of Currently running task		
RETURN

